#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <getopt.h>
#include <time.h>
#include <err.h>

#include "simulation.h"
#include "arg.h"

#define EPS 1e-12

/* depth accuracy criteria in meter for solver */
#define TOL 1e-10
#define MAX_ITER 100

/* uncomment to print time spent per time step to stdout */
/* #define BENCHMARK_PERFORMANCE */

#ifndef M_PI
#define M_PI 3.1415926535897932384626433832795028841971693993751058209749445923078164062
#endif

char *argv0;

static void
usage(void)
{
	fprintf(stderr, "usage: %s "
	        "[-h] "
	        "[-d grain-size] "
	        "[-i fluid-viscosity] "
	        "[-k fluid-permeability] "
	        "[-P grain-compressibility] "
	        "[-p grain-porosity] "
	        "[-s applied-shear-vel] "
	        "[-v] "
	        "\n", argv0);
	exit(1);
}

static double
deborah_number(const struct simulation *sim)
{
	return sim->alpha * sim->mu_f * sim->d * sim->v_x_fix
		/ (sim->k[0] * (1.0 - sim->phi[0]));
}

int
main(int argc, char *argv[])
{
	int i;
	double new_phi, new_k;
	struct simulation sim;

#ifdef BENCHMARK_PERFORMANCE
	clock_t t_begin, t_end;
	double t_elapsed;

#endif

#ifdef __OpenBSD__
	if (pledge("stdio", NULL) == -1)
		err(2, "pledge failed");
#endif

	init_sim(&sim);
	new_phi = sim.phi[0];
	new_k = sim.k[0];

	ARGBEGIN {
	case 'd':
		sim.d = atof(EARGF(usage()));
		break;
	case 'h':
		usage();
		break;
	case 'i':
		sim.mu_f = atof(EARGF(usage()));
		break;
	case 'k':
		new_k = atof(EARGF(usage()));
		break;
	case 'P':
		sim.alpha = atof(EARGF(usage()));
		break;
	case 'p':
		new_phi = atof(EARGF(usage()));
		break;
	case 's':
		sim.v_x_fix = atof(EARGF(usage()));
		break;
	case 'v':
		printf("%s-" VERSION "\n", argv0);
		return 0;
		break;
	default:
		usage();
	} ARGEND;

	if (argc > 0)
		usage();

	sim.nz = 2;
	prepare_arrays(&sim);

	if (!isnan(new_phi))
		for (i = 0; i < sim.nz; ++i)
			sim.phi[i] = new_phi;
	if (!isnan(new_k))
		for (i = 0; i < sim.nz; ++i)
			sim.k[i] = new_k;

	check_simulation_parameters(&sim);

#ifdef BENCHMARK_PERFORMANCE
	t_begin = clock();
#endif

	printf("%.17g\n", deborah_number(&sim));

#ifdef BENCHMARK_PERFORMANCE
	t_end = clock();
	t_elapsed = (double) (t_end - t_begin) / CLOCKS_PER_SEC;
	printf("time spent = %g s\n", t_elapsed);
#endif

	free_arrays(&sim);

	return 0;
}
